
.include "m8def.inc"

.def temp_0	=r12
.def temp_1	=r13
.def temp_2	=r14
.def temp_3	=r15
.def temp	=r16
.def data	=r17
.def count	=r18
.def low_reg	=r24
.def high_reg	=r25

.equ DIR_LCD	=DDRD
.equ PORT_LCD	=PORTD
.equ PIN_LCD	=PIND
.equ CTRL_LCD	=PORTC

.equ DB4	=PD4
.equ DB5	=PD5
.equ DB6	=PD6
.equ DB7	=PD7

.equ RS		=PC1
.equ RW		=PC2
.equ E		=PC3

.equ BAUD	=19200
.equ Fosc	=16000000

.equ const5us	=10-5
.equ const50us	=100-5
.equ const60us	=120-5
.equ const65us	=130-5
.equ const100us	=200-5
.equ const500us	=125-2

.equ const1ms	=(Fosc/1000-8)/4
.equ const50ms	=50
.equ const100ms	=100
.equ const500ms	=500
.equ const1s	=1000

.macro USART_TX
loop_transmit:
	sbis UCSRA, UDRE
	rjmp loop_transmit
	out UDR, data
.endm

.macro USART_RX
loop_recive:
	sbis UCSRA, RXC
	rjmp loop_recive
.endm



.DSEG
	.org SRAM_START


.CSEG

	.org 0
	rjmp begin

	.org INT_VECTORS_SIZE
begin:
	cli

	ldi temp, high(RAMEND)
	out SPH, temp
	ldi temp, low(RAMEND)
	out SPL, temp		;   
	
	ldi temp, (1<<WDCE)|(1<<WDE)
	out WDTCR, temp
	ldi temp, (0<<WDE)
	out WDTCR, temp		;  WDT

	ldi temp, 0b00111111
	out PORTB, temp
	ldi temp, 0b01110000
	out PORTC, temp
	ldi temp, 0b11111111
	out PORTD, temp

	ldi temp, 0b00000000
	out DDRB, temp	;   B
	ldi temp, 0b00001110
	out DDRC, temp	;   C
	ldi temp, 0b00000010
	out DDRD, temp	;   D

	in temp, MCUCR
	sbr temp, (1<<SE)
	cbr temp, (1<<SM0)|(1<<SM1)|(1<<SM2)
	out MCUCR, temp	;   sleep

	sei

	rcall InitLCD

	in temp, DIR_LCD
	ori temp, (1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4)
	out DIR_LCD, temp

	cbi CTRL_LCD, RS
	ldi data, 0b10000000 | 0x00
	rcall WriteLCD
	sbi CTRL_LCD, RS

	ldi data, 't'
	rcall WriteLCD
	ldi data, 'e'
	rcall WriteLCD
	ldi data, 's'
	rcall WriteLCD
	ldi data, 't'
	rcall WriteLCD
	ldi data, ' '
	rcall WriteLCD
	ldi data, '1'
	rcall WriteLCD

	cbi CTRL_LCD, RS
	ldi data, 0b10000000 | 0x40
	rcall WriteLCD
	sbi CTRL_LCD, RS

	ldi data, 't'
	rcall WriteLCD
	ldi data, 'e'
	rcall WriteLCD
	ldi data, 's'
	rcall WriteLCD
	ldi data, 't'
	rcall WriteLCD
	ldi data, ' '
	rcall WriteLCD
	ldi data, '2'
	rcall WriteLCD

	cbi CTRL_LCD, RS
	ldi data, 0b10000000 | 0x00
	rcall WriteLCD
	sbi CTRL_LCD, RS

	in temp, PORT_LCD
	ori temp, (1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4)
	out PORT_LCD, temp
	in temp, DIR_LCD
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	out DIR_LCD, temp

work:
	rcall reset_1wire
	ldi data, 0xCC
	rcall write_byte
	ldi data, 0x44
	rcall write_byte
wait:
	rcall read
	lsr temp_0
	brcc wait

	rcall reset_1wire
	ldi data, 0xCC
	rcall write_byte
	ldi data, 0xBE
	rcall write_byte
	rcall read_byte
	mov temp_2, data
	rcall read_byte
	mov temp_3, data
	rcall read_byte
	rcall read_byte
	rcall read_byte
	rcall read_byte
	rcall read_byte
	rcall read_byte
	rcall read_byte

	in temp, DIR_LCD
	ori temp, (1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4)
	out DIR_LCD, temp
	cbi CTRL_LCD, RS

	ldi data, 0b00000010
	rcall WriteLCD

	sbi CTRL_LCD, RS

	ldi data, ' '
	tst temp_3
	breq plus
	ldi data, '-'
	neg temp_2
plus:
	rcall WriteLCD

	mov temp, temp_2
	lsr temp

	rcall dsat_ASCII

	mov data, temp_1
	cpi data, '0'
	breq zero
	rcall WriteLCD
zero:
	mov data, temp_0
	rcall WriteLCD
	ldi data, '.'
	rcall WriteLCD
	sbrs temp_2, 0
	ldi data, '0'
	sbrc temp_2, 0
	ldi data, '5'
	rcall WriteLCD
	ldi data, ' '
	rcall WriteLCD
	ldi data, 0xDF
	rcall WriteLCD
	ldi data, 'C'
	rcall WriteLCD
	ldi data, ' '
	rcall WriteLCD

	in temp, PORT_LCD
	ori temp, (1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4)
	out PORT_LCD, temp
	in temp, DIR_LCD
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	out DIR_LCD, temp

	rjmp work

reset_1wire:
	sbi DDRC, PC0
	rcall timer500us
	cbi DDRC, PC0
	rcall timer500us
	ret

write_byte:
	ldi count, 8
loop_write_byte:
	sbrs data, 0
	rcall write0
	sbrc data, 0
	rcall write1
	lsr data
	dec count
	brne loop_write_byte
	ret

write0:
	sbi DDRC, PC0
	rcall timer65us
	cbi DDRC, PC0
	rcall timer5us
	ret

write1:
	sbi DDRC, PC0
	rcall timer5us
	cbi DDRC, PC0
	rcall timer65us
	ret

read_byte:
	ldi count, 8
loop_read_byte:
	rcall read
	lsr temp_0
	ror data
	dec count
	brne loop_read_byte
	ret

read:
	sbi DDRC, PC0
	rcall timer5us
	cbi DDRC, PC0
	rcall timer5us
	in temp_0, PINC
	rcall timer60us
	ret

InitLCD:
	push temp
	in temp, SREG
	push temp
	push data

	cli
	rcall timer50ms
	cbi CTRL_LCD, RS
	cbi CTRL_LCD, RW
	in temp, DIR_LCD
	ori temp, (1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4)
	out DIR_LCD, temp
	sbi CTRL_LCD, E
	in temp, PORT_LCD
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	ori temp, (0<<DB7)|(0<<DB6)|(1<<DB5)|(1<<DB4)
	out PORT_LCD, temp
	nop
	nop
	nop
	nop
	cbi CTRL_LCD, E
	rcall timer50us
	sbi CTRL_LCD, E
	in temp, PORT_LCD
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	ori temp, (0<<DB7)|(0<<DB6)|(1<<DB5)|(1<<DB4)
	out PORT_LCD, temp
	nop
	nop
	nop
	nop
	cbi CTRL_LCD, E
	rcall timer50us
	sbi CTRL_LCD, E
	in temp, PORT_LCD
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	ori temp, (0<<DB7)|(0<<DB6)|(1<<DB5)|(1<<DB4)
	out PORT_LCD, temp
	nop
	nop
	nop
	nop
	cbi CTRL_LCD, E
	rcall timer50us
	sbi CTRL_LCD, E
	in temp, PORT_LCD
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	ori temp, (0<<DB7)|(0<<DB6)|(1<<DB5)|(0<<DB4)
	out PORT_LCD, temp
	nop
	nop
	nop
	nop
	cbi CTRL_LCD, E
	rcall timer50us

	ldi data, 0b00101000
	rcall WriteLCD
	ldi data, 0b00001000
	rcall WriteLCD
	ldi data, 0b00000001
	rcall WriteLCD
	ldi data, 0b00000110
	rcall WriteLCD

	ldi data, 0b00001100
	rcall WriteLCD

	in temp, PORT_LCD
	ori temp, (1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4)
	out PORT_LCD, temp
	in temp, DIR_LCD
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	out DIR_LCD, temp

	pop data
	pop temp
	out SREG, temp
	pop temp
	ret


WriteLCD:
	push temp
	in temp, SREG
	push temp
	push data
	push data
	rcall WaitLCD
	sbi CTRL_LCD, E
	cli
	in temp, PORT_LCD
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	andi data, (1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4)
	or temp, data
	out PORT_LCD, temp
	sei
	nop
	cbi CTRL_LCD, E
	nop
	nop
	nop
	nop
	nop
	pop data
	swap data
	sbi CTRL_LCD, E
	cli
	in temp, PORT_LCD
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	andi data, (1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4)
	or temp, data
	out PORT_LCD, temp
	sei
	nop
	cbi CTRL_LCD, E
	pop data
	pop temp
	out SREG, temp
	pop temp
	ret

WaitLCD:
	push temp
	in temp, SREG
	push temp
	push data
	cli
	in temp, PORT_LCD
	push temp
	ori temp, (1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4)
	out PORT_LCD, temp
	in temp, DIR_LCD
	push temp
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	out DIR_LCD, temp
	sei
	in temp, CTRL_LCD
	push temp
	cbi CTRL_LCD, RS
	sbi CTRL_LCD, RW
loop_WaitLCD:
	sbi CTRL_LCD, E
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	in temp, PIN_LCD
	cbi CTRL_LCD, E
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	sbi CTRL_LCD, E
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	cbi CTRL_LCD, E
	nop
	nop
	nop
	nop
	nop
	andi temp, (1<<PD7)
	brne loop_WaitLCD
	cli
	in temp, CTRL_LCD
	pop data
	andi temp, ~((1<<RS)|(1<<RW))
	andi data, (1<<RS)|(1<<RW)
	or temp, data
	out CTRL_LCD, temp
	in temp, DIR_LCD
	pop data
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	andi data, (1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4)
	or temp, data
	out DIR_LCD, temp
	in temp, PORT_LCD
	pop data
	andi temp, ~((1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4))
	andi data, (1<<DB7)|(1<<DB6)|(1<<DB5)|(1<<DB4)
	or temp, data
	out PORT_LCD, temp
	pop data
	pop temp
	out SREG, temp
	pop temp
	ret

dsat_ASCII:
	clr temp_0
	clr temp_1
loop2:
	subi temp, 10
	brcs end2
	inc temp_1
	rjmp loop2
end2:
	subi temp, -10
	mov temp_0, temp
	ldi temp, 0x30
	add temp_0, temp
	add temp_1, temp
	ret

timer5us:
	ldi temp, const5us
	rcall timer2
	ret

timer50us:
	ldi temp, const50us
	rcall timer2
	ret

timer60us:
	ldi temp, const60us
	rcall timer2
	ret

timer65us:
	ldi temp, const65us
	rcall timer2
	ret

timer100us:
	ldi temp, const100us
	rcall timer2
	ret


timer2:
	out OCR2, temp

	clr temp
	out TCNT2, temp
	ldi temp, (1<<OCF2)
	out TIFR, temp
	ldi temp, (1<<PSR2)
	out SFIOR, temp
	ldi temp, (1<<WGM21)|(1<<CS21)
	out TCCR2, temp

loop_timer2:
	in temp, TIFR
	andi temp, (1<<OCF2)
	breq loop_timer2
	out TIFR, temp

	clr temp
	out TCCR2, temp
	out TCNT2, temp

	nop

	ret


timer500us:
	clr temp
	out TCNT2, temp
	ldi temp, const500us
	out OCR2, temp
	ldi temp, (1<<OCF2)
	out TIFR, temp
	ldi temp, (1<<PSR2)
	out SFIOR, temp
	ldi temp, (1<<WGM21)|(1<<CS22)
	out TCCR2, temp

loop_timer500us_1:
	in temp, TIFR
	andi temp, (1<<OCF2)
	breq loop_timer500us_1
	out TIFR, temp

	clr temp
	out TCCR2, temp
	out TCNT2, temp

	ldi temp, 13
loop_timer500us_2:
	dec temp
	brne loop_timer500us_2
	nop

	ret


timer1ms:
	ldi low_reg, low(const1ms)
	ldi high_reg, high(const1ms)
loop_timer1ms:
	sbiw high_reg:low_reg, 1
	brne loop_timer1ms
	ret

timer50ms:
	ldi XL, low(const50ms)
	ldi XH, high(const50ms)
loop_timer50ms:
	rcall timer1ms
	sbiw XH:XL, 1
	brne loop_timer50ms
	ret


#ifdef DEBUG
	rcall printf
	.db 0x0D, 0x0A, "OFF = 0x", '\0', '\0'
	mov data, XH
	rcall printf_hex
	mov data, XL
	rcall printf_hex
	rcall printf
	.db 0x0D, 0x0A, "ON = 0x", '\0'
	mov data, high_reg
	rcall printf_hex
	mov data, low_reg
	rcall printf_hex
	rcall printf
	.db 0x0D, 0x0A, '\0', '\0'


timer100ms:
	ldi XL, low(const100ms)
	ldi XH, high(const100ms)
loop_timer100ms:
	rcall timer1ms
	sbiw XH:XL, 1
	brne loop_timer100ms
	ret


timer500ms:
	ldi XL, low(const500ms)
	ldi XH, high(const500ms)
loop_timer500ms:
	rcall timer1ms
	sbiw XH:XL, 1
	brne loop_timer500ms
	ret


timer1s:
	ldi XL, low(const1s)
	ldi XH, high(const1s)
loop_timer1s:
	rcall timer1ms
	sbiw XH:XL, 1
	brne loop_timer1s
	ret


printf:
	pop ZH
	pop ZL
	push temp
	push data
	in temp, SREG
	push temp
	lsl ZL
	rol ZH
loop_printf:
	lpm data, Z+
	tst data
	brne metka_printf
	lsr ZH
	ror ZL
	pop temp
	out SREG, temp
	pop data
	pop temp
	ijmp
metka_printf:
	USART_TX
	rjmp loop_printf


printf_hex:
	push temp
	in temp, SREG
	push temp
	push data
	swap data
	andi data, 0x0F
	cpi data, 0x0A
	brlo metka_transmit_hex_0
	subi data, (~0x07+1)
metka_transmit_hex_0:
	subi data, (~0x30+1)
	USART_TX
	pop data
	push data
	andi data, 0x0F
	cpi data, 0x0A
	brlo metka_transmit_hex_1
	subi data, (~0x07+1)
metka_transmit_hex_1:
	subi data, (~0x30+1)
	USART_TX
	pop data
	pop temp
	out SREG, temp
	pop temp
	ret
#endif

.exit
